-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

local ILVL_ACTIVEHEIRLOOM = -1;
local LOWLEVEL_MULTIPLIER = 0.8;
local MAX_PET_LEVEL = 25;

-- Names of the equipment slots
local equipSlotNames =
{
  [INVSLOT_HEAD    ] = "Head",
  [INVSLOT_NECK    ] = "Neck",
  [INVSLOT_SHOULDER] = "Shoulder",
  [INVSLOT_CHEST   ] = "Chest",
  [INVSLOT_WAIST   ] = "Waist",
  [INVSLOT_LEGS    ] = "Legs",
  [INVSLOT_FEET    ] = "Feet",
  [INVSLOT_WRIST   ] = "Wrist",
  [INVSLOT_HAND    ] = "Hands",
  [INVSLOT_FINGER1 ] = "Finger0",
  [INVSLOT_FINGER2 ] = "Finger1",
  [INVSLOT_TRINKET1] = "Trinket0",
  [INVSLOT_TRINKET2] = "Trinket1",
  [INVSLOT_BACK    ] = "Back",
  [INVSLOT_MAINHAND] = "MainHand",
  [INVSLOT_OFFHAND ] = "SecondaryHand",
};

-- Info for equipped items
local equippedItems =
{
  [INVSLOT_HEAD    ] = {id = 0, iLvl = 0},
  [INVSLOT_NECK    ] = {id = 0, iLvl = 0},
  [INVSLOT_SHOULDER] = {id = 0, iLvl = 0},
  [INVSLOT_CHEST   ] = {id = 0, iLvl = 0},
  [INVSLOT_WAIST   ] = {id = 0, iLvl = 0},
  [INVSLOT_LEGS    ] = {id = 0, iLvl = 0},
  [INVSLOT_FEET    ] = {id = 0, iLvl = 0},
  [INVSLOT_WRIST   ] = {id = 0, iLvl = 0},
  [INVSLOT_HAND    ] = {id = 0, iLvl = 0},
  [INVSLOT_FINGER1 ] = {id = 0, iLvl = 0},
  [INVSLOT_FINGER2 ] = {id = 0, iLvl = 0},
  [INVSLOT_TRINKET1] = {id = 0, iLvl = 0},
  [INVSLOT_TRINKET2] = {id = 0, iLvl = 0},
  [INVSLOT_BACK    ] = {id = 0, iLvl = 0},
  [INVSLOT_MAINHAND] = {id = 0, iLvl = 0},
  [INVSLOT_OFFHAND ] = {id = 0, iLvl = 0},
};

-- Stats for equipped slots
local equippedStats =
{
  count   = 0,
  lowIlvl = 0,
  empty   = 0,
};

-- Auction sub-categories for 2H items that aren't actually flagged as 2H
local equippedTwoHanded =
{
  ["Bows"     ] = true,
  ["Crossbows"] = true,
  ["Guns"]      = true,
};

-- Item IDs for all know lances
local tournamentLances =
{
  [46069] = true, -- Alliance Lance
  [46070] = true, -- Horde Lance
  [46106] = true, -- Argent Lance
  [52716] = true  -- Twilight Firelance
};

-- Info for Pet Battle items that grant an XP bonus
local petBattleItems =
{
  {id = 92738, bonus = 10, slot = INVSLOT_HEAD} -- Safari Hat
};

-------------------------------------------------------------------------------
--                L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to scan the tooltip for the specified item to get the maximum
-- character level it will scale to.
--
local function getHeirloomMaxLevel(itemId)
  -- Local Variables
  local numLines;
  local maxLevel = 0;

  -- Create the tooltip, if required
  if (NigglesEquippedTooltip == nil) then
    CreateFrame("GameTooltip", "NigglesEquippedTooltip",
      UIParent, "GameTooltipTemplate");
    NigglesEquippedTooltip:SetOwner(UIParent, "ANCHOR_NONE")
  end

  -- Set the tooltip
  NigglesEquippedTooltip:ClearLines();
  NigglesEquippedTooltip:SetItemByID(itemId);

  -- Scan the lines of the tooltip for the level range
  numLines = NigglesEquippedTooltip:NumLines();
  for lineIdx = numLines, 1, -1 do
    _, _, maxLevel = string.find(
      _G["NigglesEquippedTooltipTextLeft"..lineIdx]:GetText(),
      L["HeirloomLevelPattern"]);
    if (maxLevel ~= nil) then
      break;
    end
  end

  return maxLevel;
end

-------------------------------------------------------------------------------
--               G  L  O  B  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to check if Pet Battle items aren't equipped at start of a battle.
--
-- NOTE: This function will need enhancement if Blizzard ever add items for
--       ring, trinket, main or off-hand slots.
--
function NigglesEquipmentCheckPetBattle()
  -- Local Variables
  local bestUnequipped = {};
  local itemLink;
  local itemTexture;
  local numUnequipped = 0;
  local string = "";
  local nonMaxLevel = false;

  -- Work out if there are any pets that aren't max level
  for idx = 1, C_PetBattles.GetNumPets(1) do
    if (C_PetBattles.GetLevel(1, idx) < MAX_PET_LEVEL) then
      nonMaxLevel = true;
      break;
    end
  end

  -- Check if there are any pets that aren't max level
  if (nonMaxLevel) then
    -- Initialise the best unequipped table
    for _, itemInfo in pairs(petBattleItems) do
      bestUnequipped[itemInfo.slot] = {bonus = 0, id = 0};
    end

    -- Check if the best Pet Battle equipment owned is equipped
    for _, itemInfo in ipairs(petBattleItems) do
      if ((itemInfo.bonus > bestUnequipped[itemInfo.slot].bonus) and
          (not IsEquippedItem(itemInfo.id)) and
          ((GetItemCount(itemInfo.id, false) > 0) or
           (GetItemCount(itemInfo.id, true) > 0))) then
        if (bestUnequipped[itemInfo.slot].id == 0) then
          numUnequipped = numUnequipped+1;
        end
        bestUnequipped[itemInfo.slot].id    = itemInfo.id;
        bestUnequipped[itemInfo.slot].bonus = itemInfo.bonus;
      end
    end

    -- Check if there are any items that should be equipped
    if (numUnequipped > 0) then
      -- Create the Niggles warning popup, if required
      if (Niggles.warningPopup == nil) then
        CreateFrame("Frame", "NigglesWarningPopup", UIParent,
          "NigglesWarningPopupTemplate");
      end

      -- Create the string for the unequipped items
      for _, unequippedInfo in pairs(bestUnequipped) do
        if (unequippedInfo.id ~= 0) then
          _, itemLink, _, _, _, _, _, _, _, itemTexture =
            GetItemInfo(unequippedInfo.id);
          -- NOTE: The '|r' is done after the '\n' to overcome a Blizzard bug
          string  = string.."\124T"..itemTexture..":0:0:0:-8\124t "..
            itemLink.."\n|r";
        end
      end

      -- Set the info to be displayed
      NigglesWarningPopup.text:SetText(L["PetBattleUnequipped"]);
      NigglesWarningPopup.list:SetText(string);
      NigglesWarningPopup.list:Show();
      NigglesWarningPopup.item:Hide();

      -- Display the frame
      StaticPopupSpecial_Show(NigglesWarningPopup);
    end
  end

  return;
end

--
-- Function to get the number of empty slots
--
function NigglesEquipmentGetEmptySlots()
  return equippedStats.empty;
end

--
-- Function to get the number of low level items equipped
--
function NigglesEquipmentGetLowIlvl()
  return equippedStats.lowIlvl;
end

--
-- Function to check if a lance is equipped
--
function NigglesEquipmentIsLance(itemId)
  return (tournamentLances[itemId] ~= nil);
end

--
-- Function to scan all equipped items. It works out an average ilvl, ignoring
-- active heirlooms, as well as counting the number of empty slots.
--
function NigglesEquipmentScan()
  -- Local Variables
  local averageIlvl = 0;
  local itemIlvl;
  local itemId;
  local itemQuality;
  local itemSubCat;
  local itemType;
  local lowIlvl;
  local infoAvailable = true;

  -- Reset the stats for the equipped items
  equippedStats.count   = 0;
  equippedStats.lowIlvl = 0;
  equippedStats.empty   = 0;

  -- Scan all equipment slots
  for slotId, slotInfo in pairs(equippedItems) do
    itemId = GetInventoryItemID("player", slotId);
    if (itemId == nil) then
      slotInfo.id         = 0;
      slotInfo.iLvl       = 0;
      equippedStats.empty = equippedStats.empty+1;
    else
      -- Check if the item has changed since the last scan
      if (itemId ~= slotInfo.id) then
        -- Get the item's info
        _, _, itemQuality, itemIlvl = GetItemInfo(itemId);
        if (itemQuality ~= nil) then
          slotInfo.id   = itemId;
          slotInfo.iLvl = itemIlvl;

          -- Check if the item is an active heirloom, giving two level of grace
          if ((itemQuality == 7) and
              (getHeirloomMaxLevel(itemId)+2 >= UnitLevel("player"))) then
            slotInfo.iLvl = ILVL_ACTIVEHEIRLOOM;
          end
        else
          slotInfo.id   = 0;
          slotInfo.iLvl = 0;
          infoAvailable = false;
          break;
        end
      end

      -- Update the info for calculating the average item level
      if ((infoAvailable) and
          (slotInfo.iLvl ~= ILVL_ACTIVEHEIRLOOM)) then
        averageIlvl         = averageIlvl+slotInfo.iLvl;
        equippedStats.count = equippedStats.count+1;
      end
    end
  end
  
  -- Check item info could be obtained for all items
  if (infoAvailable) then
    -- Adjust the number of empty slots to account for 2H weapons
    if ((equippedItems[INVSLOT_MAINHAND].id ~= 0) and
        (equippedItems[INVSLOT_OFFHAND].id  == 0)) then
      _, _, _, _, _, _, itemSubCat, _, itemType =
        GetItemInfo(equippedItems[INVSLOT_MAINHAND].id);
      if ((itemType == "INVTYPE_2HWEAPON") or 
          (equippedTwoHanded[itemSubCat] ~= nil)) then
        equippedStats.empty = equippedStats.empty-1;
      end
    end

    -- Work out the average item level
    if (equippedStats.count > 0) then
      averageIlvl = averageIlvl/equippedStats.count;
    else
      averageIlvl = 0;
    end

    -- Work out how many low level items are equipped
    lowIlvl = averageIlvl*LOWLEVEL_MULTIPLIER;
    for _, slotInfo in pairs(equippedItems) do
      if ((slotInfo.id   > 0) and
          (slotInfo.iLvl ~= ILVL_ACTIVEHEIRLOOM) and
          (slotInfo.iLvl < lowIlvl)) then
        equippedStats.lowIlvl = equippedStats.lowIlvl+1;
      end
    end
  else
    -- Reset the stats for the equipped items
    equippedStats.count   = 0;
    equippedStats.lowIlvl = 0;
    equippedStats.empty   = 0;

    -- Create a timer to scan the equipment again
    NigglesTimerCreate("EquipmentScan", NigglesEquipmentScan,
      nil, nil, 1.0, false);
  end

  -- Update the low level item indicators
  if ((NigglesDBC.settings.UiLowLevelItems) and
      (equippedStats.lowIlvl > 0)) then
    lowIlvl = averageIlvl*LOWLEVEL_MULTIPLIER;
    for slotId, slotInfo in pairs(equippedItems) do
      if ((slotInfo.id   > 0) and
          (slotInfo.iLvl ~= ILVL_ACTIVEHEIRLOOM) and
          (slotInfo.iLvl < lowIlvl)) then
        _G["NigglesLowIlvl"..slotId]:Show();
      else
        _G["NigglesLowIlvl"..slotId]:Hide();
      end
    end
  else
    for slotId, _ in pairs(equippedItems) do
      _G["NigglesLowIlvl"..slotId]:Hide();
    end
  end

  return;
end

--
-- Function to initialise the Niggles Equipment functions
--
function NigglesEquipmentInit()
  -- Create the frames used to indicate low level items
  -- NOTE: A font string can't simply be created as a child of the button
  --       directly because it can't be placed above the cooldown.
  for slotId, slotName in pairs(equipSlotNames) do
    CreateFrame("Frame", "NigglesLowIlvl"..slotId,
      _G["Character"..slotName.."Slot"], "NigglesLowIlvlTemplate");
  end

  -- Hook the function used to sell items
  hooksecurefunc("UseContainerItem", NigglesEquipmentOnItemSale);

  return;
end

--
-- Function called when an item is being sold
--
function NigglesEquipmentOnItemSale(container, slot, target)
  -- Local Variables
  local popup;

  -- Check if the item is being sold
  if ((MerchantFrame:IsShown()) and
      (MerchantFrame.selectedTab == 1)) then
    -- Check if the item is part of a set and a warning should be displayed
    if ((NigglesDBC.settings.WarningEquipSetItem) and
        (GetContainerItemEquipmentSetInfo(container, slot))) then
      local itemTexture, itemCount, _, itemQuality, _, _, itemLink =
        GetContainerItemInfo(container, slot);
      local _, _, itemName = string.find(itemLink, "|h%[(.*)%]|h|r$");
      
      -- Create the Niggles warning popup, if required
      if (NigglesWarningPopup == nil) then
        CreateFrame("Frame", "NigglesWarningPopup", UIParent,
          "NigglesWarningPopupTemplate");
      end
      popup = NigglesWarningPopup;

      -- Set the info to be displayed
      popup.text:SetText(L["EquipSetItemSold"]);
      popup.list:Hide();
      popup.item.container = container;
      popup.item.slot      = slot;
      popup.item.icon:SetTexture(itemTexture);
      popup.item.text:SetTextColor(
        ITEM_QUALITY_COLORS[itemQuality].r,
        ITEM_QUALITY_COLORS[itemQuality].g,
        ITEM_QUALITY_COLORS[itemQuality].b,
        1);
      popup.item.text:SetText(itemName);
      popup.item:Show();

      -- Display the frame
      StaticPopupSpecial_Show(NigglesWarningPopup);
    end
  end
  
  return;
end
